//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Sound_Library v1.4
//  Example5 - Notifications
//
// Date: 17th May, 2001
// Author: Darren John Dwyer
// Copyright: (c) 2001 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to setup Sound Buffer Notifications at specific
// byte-offsets within a DirectSound Buffer, then how to respond to the
// TDx_SoundNotify::OnNotify() and TDSBPositionNotify::OnNotify() events.
//
// Note: It appears DirectSound has a maximum limit of 60 notification
// positions per buffer.
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSCaps"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
#pragma link "TDx_SoundNotify"
#pragma link "TDSBPositionNotify"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
int INDENT = 0;
#define Msg(A) 	try {\
					AnsiString ssss = "";\
                    for (int nnnn=0;nnnn<INDENT*4;nnnn++) ssss = ssss + " ";\
                    ssss = ssss + (A);\
                    if (StatusMemo)\
                        if (StatusMemo->Lines)\
                            {\
                            if (StatusMemo->Lines->Text.Length()>10000) StatusMemo->Lines->Clear();\
							StatusMemo->Lines->Add( ssss );\
                            }\
                    }\
				catch (Exception &E)\
                	{\
                    if (StatusMemo)\
                        if (StatusMemo->Lines)\
                    		StatusMemo->Lines->Clear();\
                    }\
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
KnownDevices = new TStringList();
KnownModules = new TStringList();
KnownGuids = new TList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (KnownGuids) delete KnownGuids;
if (KnownModules) delete KnownModules;
if (KnownDevices) delete KnownDevices;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListBox1DblClick(TObject *Sender)
{
PlayButtonClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayButtonClick(TObject *Sender)
{
Msg(" ");
Msg("<< Play Button was Clicked >>");

INDENT++;

if (Edit1->Text.Pos("*")!=0)
	{
    Msg( "    Cannot play *.WAV" );
    INDENT--;
    return;
    }

if ((Edit1->Text!=LastFileName) || !Dx_SoundBuffer1->Created)
	{
    if (Dx_SoundBuffer1->Created)
    	{
        Msg("Destroying existing buffer...");
        Dx_SoundBuffer1->Destroy();
        }

	Msg("Creating From File( \""+Edit1->Text+"\" ) ...");
	if (Dx_SoundBuffer1->CreateFromFile( Edit1->Text, DSBufferDesc1, Dx_Sound1 ))
        LastFileName = Edit1->Text;
    }
else
    Msg("Restarting \""+LastFileName+"\" ...");

if (Dx_SoundBuffer1->Created)
	{
	if (LoopingCheckBox->Checked)
    	{
        Dx_SoundBuffer1->Play(0,DSBPLAY_LOOPING);
	    if (Dx_SoundBuffer1->Playing) PlayingLabel->Caption = "Buffer is Looping...";
        }
    else
    	{
        Dx_SoundBuffer1->Play(0,0);
	    if (Dx_SoundBuffer1->Playing) PlayingLabel->Caption = "Buffer is Playing...";
        }
    }

INDENT--;
Msg("<< Play Button has Finished >>");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Create(TObject *Sender)
{
Msg("TDx_SoundBuffer was created ok");

INDENT++;

BufferSizeLabel->Caption = "Buffer Size: "+IntToStr(DSBufferDesc1->BufferBytes)+" bytes";

Msg("Creating Buffer Notifications");

int one_percent = DSBufferDesc1->BufferBytes/100;

DSBPositionNotify1->ArraySize = 50;
for (int i=0;i<49;i++) DSBPositionNotify1->Offset[i] = 2*i*one_percent;
DSBPositionNotify1->Offset[49] = DSBPN_OFFSETSTOP;

if (Dx_SoundNotify1->Create( Dx_SoundBuffer1 ))
    if (Dx_SoundNotify1->SetNotificationPositions( DSBPositionNotify1->ArraySize, DSBPositionNotify1 ))
        {
        Msg("Notification Positions set ok");
        }
    else
        Msg("Notification Positions not set!");

INDENT--;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundNotify1Create(TObject *Sender)
{
Msg("TDx_SoundNotify was created ok");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopButtonClick(TObject *Sender)
{
Msg("");
Msg("<< Stop Button was Clicked >>");

INDENT++;

if (Dx_SoundBuffer1->Created)
	{
	Msg("About to destroy TDx_SoundBuffer");
    Dx_SoundBuffer1->Destroy();
    }

INDENT--;

Msg("<< Stop Button has Finished >>");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Destroy(TObject *Sender)
{
Msg("TDx_SoundBuffer is being Destroyed..." );

INDENT++;

if (Dx_SoundBuffer1->Playing)
	{
	Msg("Stopping buffer playback..." );
    Dx_SoundBuffer1->Stop();
    }

if (Dx_SoundNotify1->Created)
	{
	Msg("About to Destroy TDx_SoundNotify");
    Dx_SoundNotify1->Destroy();
    }

INDENT--;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundNotify1Destroy(TObject *Sender)
{
Msg("TDx_SoundNotify is being Destroyed..." );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
StopButtonClick(Sender);
Close();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DSBPositionNotify1Notify(TObject *pSender,
    int pNotification)
{
Msg("");
Msg(">> Inside the TDSBPositionNotify::OnNotify() Event..." );

INDENT++;

// if the Dx_SoundNotify1::LocalNotify property is set to true,
// this event is called with the relevant notification position

// if the Dx_SoundNotify1::LocalNotify property is set to false,
// the relevant TDx_SoundNotify::OnNotify() event will be called

NotificationLabel->Caption = "Notification: "+IntToStr(pNotification);
BufferPosLabel->Caption = "Last Pos: "+IntToStr(pNotification*2)+"%";
BufferPosLabel->Update();

if (pNotification==DSBPositionNotify1->ArraySize-1)
    {
	PlayingLabel->Caption = "Buffer is Stopping...";
    BufferPosLabel->Caption = "Last Pos: DSPN_OFFSETSTOP";
    StopButtonClick(this);
    }
Msg("Handled Notification #"+NotificationLabel->Caption+", "+BufferPosLabel->Caption );

INDENT--;
Msg("");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundNotify1Notify(TObject *Sender, int pNotification)
{
Msg("");
Msg(">> Inside the TDx_SoundNotify::OnNotify() Event..." );

INDENT++;

// if the Dx_SoundNotify1::LocalNotify property is set to true,
// this event is called with the relevant notification position

// if the Dx_SoundNotify1::LocalNotify property is set to false,
// the relevant DSBPositionNotify[X]::OnNotify() events will be called

NotificationLabel->Caption = "Notification: "+IntToStr(pNotification);
BufferPosLabel->Caption = "Last Pos: "+IntToStr(pNotification*2)+"%";
BufferPosLabel->Update();

if (pNotification==DSBPositionNotify1->ArraySize-1)
    {
	PlayingLabel->Caption = "Buffer is Stopping...";
    BufferPosLabel->Caption = "Last Pos: DSPN_OFFSETSTOP";
    StopButtonClick(this);
    }
Msg("Handled Notification #"+NotificationLabel->Caption+", "+BufferPosLabel->Caption );

INDENT--;
Msg("");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
NotCreatedLabel->Visible = !Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToggleButtonClick(TObject *Sender)
{
Dx_SoundNotify1->LocalNotify = !Dx_SoundNotify1->LocalNotify;

if (Dx_SoundNotify1->LocalNotify) StatusLabel->Caption = "Events handled by TDx_SoundNotify::OnNotify()";
else StatusLabel->Caption = "Events handled by TDSBPositionNotify::OnNotify()";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Dx_SoundNotify1Error(TObject *Sender,
	AnsiString TheFunction, AnsiString TheError, AnsiString TheErrorMessage)
{
if (Visible) Msg(TheErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
Msg("In>  Dx_SoundBuffer1::OnBufferLost()");
INDENT++;

Msg("Dx_SoundBuffer1::OnBufferLost(), creating Dx_SoundBuffer1");
Dx_SoundBuffer1->CreateFromFile( Edit1->Text, DSBufferDesc1, Dx_Sound1 );

Msg("Dx_SoundBuffer1::OnBufferLost(), playing Dx_SoundBuffer1");
Dx_SoundBuffer1->Play( 0, 0 );

INDENT--; Msg("Out> Dx_SoundBuffer1::OnBufferLost()");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
DevicesListBox->Items->Clear();
if (Dx_Sound1->DSEnumerate( NULL ))
    if (KnownDevices->Count>0)
    	{
	    DevicesListBox->ItemIndex = 0;
	    Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormHide(TObject *Sender)
{
// make sure to destroy component internals before the components are destroyed.
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
Dx_Sound1->SetCooperativeLevel(Application->Handle, DSSCL_PRIORITY);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Error(TObject *Sender,
    AnsiString Function, AnsiString Error, AnsiString ErrorMessage)
{
Msg("--------------------------------------");
Msg("Dx_SoundBuffer1 Error");
Msg("--------------------------------------");
Msg("Error:    "+Error);
Msg("Error#:   "+IntToStr(Dx_SoundBuffer1->ErrorValue));
Msg("Function: "+Function);
Msg("Message:  "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("--------------------------------------");
Msg("Dx_Sound1 Error");
Msg("--------------------------------------");
Msg("Error:    "+Error);
Msg("Error#:   "+IntToStr(Dx_SoundBuffer1->ErrorValue));
Msg("Function: "+Function);
Msg("Message:  "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1DSEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverModule, void *Context,
    bool &Finished)
{
if (KnownDevices && KnownModules && KnownGuids)
	{
    KnownDevices->Add( DriverDescription );
    KnownModules->Add( DriverModule );
    KnownGuids->Add( (void*) Guid );
    DevicesListBox->Items->Add( DriverDescription + ", ("+DriverModule+")" );
    }
else
	Finished = true;
}
//---------------------------------------------------------------------------
